classdef DemCallbackEventStatusChange < autosar.bsw.ServiceImplementation
% Class to define Dem CallbackEventStatusChange
    
%   Copyright 2016 The MathWorks, Inc.

    properties(Constant, Access=private)
        DefaultClientPortName = 'CallbackEventStatusChange';
        DefaultInterfacePath = '/AUTOSAR/Services/Dem';
        InterfaceName = 'CallbackEventStatusChange';
        Operations = {'EventStatusChanged'};
        
        FunctionPrototypes = {'ERR = %s_EventStatusChanged(EventStatusOld,EventStatusNew)'};

        InputArgSpecs = {'%s.getDefaultValue, %s.getDefaultValue'}; % Dem_EventStatusExtendedType, Dem_EventStatusExtendedType
        
        OutputArgSpecs = {'uint8(1)'}; % P_Std_ReturnType
        
        EnumDatatypes = {'Enum: Dem_EventStatusExtendedType'};
        
        DatatypeMaskPromptName = {'Data type for EventStatusOld and EventStatusNew:'};
        
        FunctionPrototypeMap = containers.Map( autosar.bsw.DemCallbackEventStatusChange.Operations, ...
                                               autosar.bsw.DemCallbackEventStatusChange.FunctionPrototypes );
                                           
        InputArgSpecMap = containers.Map( autosar.bsw.DemCallbackEventStatusChange.Operations, ...
                                          autosar.bsw.DemCallbackEventStatusChange.InputArgSpecs );

        OutputArgSpecMap = containers.Map( autosar.bsw.DemCallbackEventStatusChange.Operations, ...
                                           autosar.bsw.DemCallbackEventStatusChange.OutputArgSpecs );
        
        EnumDatatypeMap = containers.Map( autosar.bsw.DemCallbackEventStatusChange.Operations, ...
                                          autosar.bsw.DemCallbackEventStatusChange.EnumDatatypes );
        
        DatatypeMaskPromptNameMap = containers.Map( autosar.bsw.DemCallbackEventStatusChange.Operations, ...
                                                    autosar.bsw.DemCallbackEventStatusChange.DatatypeMaskPromptName );

    end
    
    methods(Access=public)
        
        function this = DemCallbackEventStatusChange()
        end

        function updateFunctionCaller(this, blkPath, portName, operationName)

            % Define Dem enumeration types
            autosar.bsw.Dem_defineIntEnumTypes;
            
            functionPrototypeTemplate = this.FunctionPrototypeMap(operationName);
            functionPrototype = sprintf(functionPrototypeTemplate, portName);
            
            inputArgSpec = this.InputArgSpecMap(operationName);
            outputArgSpec = this.OutputArgSpecMap(operationName);
            if strcmp(autosar.bsw.DemCallbackEventStatusChange.getDatatypeVisibility(operationName), 'on')
                dataType = strrep(strtrim(get_param(blkPath, 'Datatype')), 'Enum: ', '');
                inputArgSpec = sprintf(inputArgSpec, dataType, dataType);
            end
            
            set_param(blkPath, 'FunctionPrototype', functionPrototype);
            set_param(blkPath, 'InputArgumentSpecifications', inputArgSpec);
            set_param(blkPath, 'OutputArgumentSpecifications', outputArgSpec);
            
        end
        
    end
    
    methods(Static, Access=public)

        function deepCopyInterface(mdlName, dstInterfacePath) %#ok<INUSD>
        % Copy interface and data types to model

            arxmlPath = fullfile(autosarroot, '+autosar', '+bsw', '+arxml');
            arxmlFiles = {fullfile(arxmlPath, 'AUTOSAR_BaseTypes.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_Type.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_MOD_PlatformTypes.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_Dem.arxml')};
                      
            autosar.bsw.ServiceImplementation.updateReferences(mdlName, arxmlFiles);
        end
        
        function defaultClientPortName = getDefaultClientPortName()
            defaultClientPortName =  autosar.bsw.DemCallbackEventStatusChange.DefaultClientPortName;
        end
        
        function dataType = getDefaultDatatype(defaultOperation)
            dataType = autosar.bsw.DemCallbackEventStatusChange.EnumDatatypeMap(defaultOperation);
        end
       
        function defaultInterfacePath = getDefaultInterfacePath()
            defaultInterfacePath =  autosar.bsw.DemCallbackEventStatusChange.DefaultInterfacePath;
        end
        
        function interfaceName = getInterfaceName()
            interfaceName =  autosar.bsw.DemCallbackEventStatusChange.InterfaceName;
        end
        
        function operations = getOperations()
            operations =  autosar.bsw.DemCallbackEventStatusChange.Operations;
        end
        
        function hastypeParameter = hasDatatypeParameter()
            hastypeParameter = true;
        end
        
        function dataTypeCallback(blkPath)
            autosar.bsw.Dem_defineIntEnumTypes;
            operation = get_param(blkPath, 'Operation');
            if strcmp(autosar.bsw.DemCallbackEventStatusChange.getDatatypeVisibility(operation), 'on')
                autosar.bsw.ServiceImplementation.dataTypeCallbackImpl(blkPath);
            end
        end
        
        function operationCallback(blkPath)
            operationValue = get_param(blkPath, 'Operation');
            autosar.bsw.ServiceImplementation.operationCallbackImpl(blkPath,...
                autosar.bsw.DemCallbackEventStatusChange.EnumDatatypeMap(operationValue),...
                autosar.bsw.DemCallbackEventStatusChange.EnumDatatypes,...
                autosar.bsw.DemCallbackEventStatusChange.DatatypeMaskPromptNameMap)
        end
        
        % returns true if we should show the Datatype parameter for the
        % specified operation
        function visibility = getDatatypeVisibility(operation)
            datatype = autosar.bsw.DemCallbackEventStatusChange.EnumDatatypeMap(operation);
            if isempty(datatype)
                visibility = 'off';
            else
                visibility = 'on';
            end
        end        
       
        function desc = getDescription()
            desc = autosar.bsw.DemDiagnosticInfo.getDescription();
        end
        
        function type = getType()
            type = 'CallbackEventStatusChangeCaller';
        end
        
        function help = getHelp()
            help = 'helpview(fullfile(docroot,''ecoder'',''helptargets.map''),''autosar_dem_callbackeventstatuschange_block'');';
        end
        
    end
    
end

% LocalWords:  bsw helptargets dem callbackeventstatuschange
