classdef DemIUMPRDenominator < autosar.bsw.ServiceImplementation
    % Class to define Dem IUMPRDenominator

    %   Copyright 2017 The MathWorks, Inc.

    properties(Constant, Access=private)
        DefaultClientPortName = 'IUMPRDenominator';
        DefaultInterfacePath = '/AUTOSAR/Services/Dem';
        InterfaceName = 'IUMPRDenominator';
        Operations = {
            'RepIUMPRDenLock'
            'RepIUMPRDenRelease'
            };

        FunctionPrototypes = {
            'ERR = %s_RepIUMPRDenLock()'
            'ERR = %s_RepIUMPRDenRelease()'
            };

        InputArgSpecs = {
            ''
            ''
            };

        OutputArgSpecs = {
            'uint8(1)'
            'uint8(1)'
            };

        FunctionPrototypeMap = ...
            containers.Map( autosar.bsw.DemIUMPRDenominator.Operations, ...
            autosar.bsw.DemIUMPRDenominator.FunctionPrototypes );

        InputArgSpecMap = ...
            containers.Map( autosar.bsw.DemIUMPRDenominator.Operations, ...
            autosar.bsw.DemIUMPRDenominator.InputArgSpecs );

        OutputArgSpecMap = ...
            containers.Map( autosar.bsw.DemIUMPRDenominator.Operations, ...
            autosar.bsw.DemIUMPRDenominator.OutputArgSpecs );

    end

    methods(Access=public)

        function this = DemIUMPRDenominator()
        end

        function updateFunctionCaller(this, blkPath, portName, operationName)

            functionPrototypeTemplate = this.FunctionPrototypeMap(operationName);
            functionPrototype = sprintf(functionPrototypeTemplate, portName);

            inputArgSpec = this.InputArgSpecMap(operationName);
            outputArgSpec = this.OutputArgSpecMap(operationName);

            set_param(blkPath, 'FunctionPrototype', functionPrototype);
            set_param(blkPath, 'InputArgumentSpecifications', inputArgSpec);
            set_param(blkPath, 'OutputArgumentSpecifications', outputArgSpec);

        end

    end

    methods(Static, Access=public)

        function deepCopyInterface(mdlName, dstInterfacePath) %#ok<INUSD>
            % Copy interface and data types to model

            arxmlPath = fullfile(autosarroot, '+autosar', '+bsw', '+arxml');
            arxmlFiles = {fullfile(arxmlPath, 'AUTOSAR_BaseTypes.arxml')
                fullfile(arxmlPath, 'AUTOSAR_Type.arxml')
                fullfile(arxmlPath, 'AUTOSAR_MOD_PlatformTypes.arxml')
                fullfile(arxmlPath, 'AUTOSAR_Dem_extras.arxml')};

            autosar.bsw.ServiceImplementation.updateReferences(mdlName, arxmlFiles);
        end

        function defaultClientPortName = getDefaultClientPortName()
            defaultClientPortName =  autosar.bsw.DemIUMPRDenominator.DefaultClientPortName;
        end

        function interfaceName = getInterfaceName()
            interfaceName =  autosar.bsw.DemIUMPRDenominator.InterfaceName;
        end

        function operations = getOperations()
            operations =  autosar.bsw.DemIUMPRDenominator.Operations;
        end

        % This block does not expose a Datatype parameter because it does
        % not use any Enum types
        function hastypeParameter = hasDatatypeParameter()
            hastypeParameter = false;
        end

        function dataTypeCallback(~)
        end

        function operationCallback(~)
        end

        function desc = getDescription()
            desc = autosar.bsw.DemDiagnosticInfo.getDescription();
        end

        function type = getType()
            type = 'DiagnosticIUMPRDenominatorCaller';
        end

        function help = getHelp()
            help = 'helpview(fullfile(docroot,''ecoder'',''helptargets.map''),''autosar_dem_diagnosticmonitor_block'');';
        end

    end

end

% LocalWords:  Prestore Prestored bsw helptargets dem diagnosticmonitor IUMPR
