classdef DemOperationCycle < autosar.bsw.ServiceImplementation
    % Class to define Dem OperationCycle

    %   Copyright 2017 The MathWorks, Inc.

    properties(Constant, Access=private)
        DefaultClientPortName = 'OperationCycle';
        DefaultInterfacePath = '/AUTOSAR/Services/Dem';
        InterfaceName = 'OperationCycle';
        Operations = {
            'SetOperationCycleState'
            'GetOperationCycleState'
            };

        FunctionPrototypes = {
            'ERR = %s_SetOperationCycleState(CycleState)'
            '[CycleState, ERR] = %s_GetOperationCycleState()'
            };

        InputArgSpecs = {
            'uint8(1)'
            ''
            };

        OutputArgSpecs = {
            'uint8(1)'
            'uint8(1),uint8(1)' % OperationCycleStateType, Std_ReturnType
            };

        EnumDatatypes = {
            'Enum: Dem_OperationCycleStateType'
            ''
            };

        DatatypeMaskPromptName = {
            'Data type for Dem_OperationCycleStateType'
            ''
            };

        FunctionPrototypeMap = ...
            containers.Map( autosar.bsw.DemOperationCycle.Operations, ...
            autosar.bsw.DemOperationCycle.FunctionPrototypes );

        InputArgSpecMap = ...
            containers.Map( autosar.bsw.DemOperationCycle.Operations, ...
            autosar.bsw.DemOperationCycle.InputArgSpecs );

        OutputArgSpecMap = ...
            containers.Map( autosar.bsw.DemOperationCycle.Operations, ...
            autosar.bsw.DemOperationCycle.OutputArgSpecs );

        EnumDatatypeMap = ...
            containers.Map( autosar.bsw.DemOperationCycle.Operations, ...
            autosar.bsw.DemOperationCycle.EnumDatatypes );

        DatatypeMaskPromptNameMap = ...
            containers.Map( autosar.bsw.DemOperationCycle.Operations, ...
            autosar.bsw.DemOperationCycle.DatatypeMaskPromptName );

    end

    methods(Access=public)

        function this = DemOperationCycle()
        end

        function updateFunctionCaller(this, blkPath, portName, operationName)

            % Define Dem enumeration types
            autosar.bsw.Dem_extras_defineIntEnumTypes;

            functionPrototypeTemplate = this.FunctionPrototypeMap(operationName);
            functionPrototype = sprintf(functionPrototypeTemplate, portName);

            inputArgSpec = this.InputArgSpecMap(operationName);
            outputArgSpec = this.OutputArgSpecMap(operationName);
            if strcmp(autosar.bsw.DemOperationCycle.getDatatypeVisibility(operationName), 'on')
                dataType = strrep(strtrim(get_param(blkPath, 'Datatype')), 'Enum: ', '');
                inputArgSpec = sprintf(inputArgSpec, dataType);
            end

            set_param(blkPath, 'FunctionPrototype', functionPrototype);
            set_param(blkPath, 'InputArgumentSpecifications', inputArgSpec);
            set_param(blkPath, 'OutputArgumentSpecifications', outputArgSpec);

        end

    end

    methods(Static, Access=public)

        function deepCopyInterface(mdlName, dstInterfacePath) %#ok<INUSD>
            % Copy interface and data types to model

            arxmlPath = fullfile(autosarroot, '+autosar', '+bsw', '+arxml');
            arxmlFiles = {fullfile(arxmlPath, 'AUTOSAR_BaseTypes.arxml')
                fullfile(arxmlPath, 'AUTOSAR_Type.arxml')
                fullfile(arxmlPath, 'AUTOSAR_MOD_PlatformTypes.arxml')
                fullfile(arxmlPath, 'AUTOSAR_Dem_extras.arxml')};

            autosar.bsw.ServiceImplementation.updateReferences(mdlName, arxmlFiles);
        end

        function defaultClientPortName = getDefaultClientPortName()
            defaultClientPortName =  autosar.bsw.DemOperationCycle.DefaultClientPortName;
        end

        function dataType = getDefaultDatatype(defaultOperation)
            dataType = autosar.bsw.DemOperationCycle.EnumDatatypeMap(defaultOperation);
        end

        function interfaceName = getInterfaceName()
            interfaceName =  autosar.bsw.DemOperationCycle.InterfaceName;
        end

        function operations = getOperations()
            operations =  autosar.bsw.DemOperationCycle.Operations;
        end

        function hastypeParameter = hasDatatypeParameter()
            hastypeParameter = true;
        end

        function dataTypeCallback(blkPath)
            autosar.bsw.Dem_extras_defineIntEnumTypes;
            operation = get_param(blkPath, 'Operation');
            if strcmp(autosar.bsw.DemOperationCycle.getDatatypeVisibility(operation), 'on')
                autosar.bsw.ServiceImplementation.dataTypeCallbackImpl(blkPath);
            end
        end

        function operationCallback(blkPath)
            operationValue = get_param(blkPath, 'Operation');
            autosar.bsw.ServiceImplementation.operationCallbackImpl(blkPath,...
                autosar.bsw.DemOperationCycle.EnumDatatypeMap(operationValue),...
                autosar.bsw.DemOperationCycle.EnumDatatypes,...
                autosar.bsw.DemOperationCycle.DatatypeMaskPromptNameMap)
        end

        % returns true if we should show the Datatype parameter for the
        % specified operation
        function visibility = getDatatypeVisibility(operation)
            datatype = autosar.bsw.DemOperationCycle.EnumDatatypeMap(operation);
            if isempty(datatype)
                visibility = 'off';
            else
                visibility = 'on';
            end
        end

        function desc = getDescription()
            desc = autosar.bsw.DemDiagnosticInfo.getDescription();
        end

        function type = getType()
            type = 'DiagnosticOperationCycleCaller';
        end

        function help = getHelp()
            help = 'helpview(fullfile(docroot,''ecoder'',''helptargets.map''),''autosar_dem_diagnosticmonitor_block'');';
        end

    end

end

% LocalWords:  Prestore Prestored bsw helptargets dem diagnosticmonitor
