classdef FiM_FunctionInhibition < autosar.bsw.ServiceImplementation
    % Class to define FiM_FunctionInhibition

    %   Copyright 2017 The MathWorks, Inc.

    properties(Constant, Access=private)
        DefaultClientPortName = 'FiM_FunctionInhibition';
        DefaultInterfacePath = '/AUTOSAR/Services/FiM';
        InterfaceName = 'FiM_FunctionInhibition';
        Operations = {
            'GetFunctionPermission'
            };

        FunctionPrototypes = {
            '[Permission,ERR] =  %s_GetFunctionPermission()'
            };

        InputArgSpecs = {
            ''
            };

        OutputArgSpecs = {
            'boolean(true), uint8(1)'
            };

        FunctionPrototypeMap = ...
            containers.Map( autosar.bsw.FiM_FunctionInhibition.Operations, ...
            autosar.bsw.FiM_FunctionInhibition.FunctionPrototypes );

        InputArgSpecMap = ...
            containers.Map( autosar.bsw.FiM_FunctionInhibition.Operations, ...
            autosar.bsw.FiM_FunctionInhibition.InputArgSpecs );

        OutputArgSpecMap = ...
            containers.Map( autosar.bsw.FiM_FunctionInhibition.Operations, ...
            autosar.bsw.FiM_FunctionInhibition.OutputArgSpecs );

    end

    methods(Access=public)

        function this = FiM_FunctionInhibition()
        end

        function updateFunctionCaller(this, blkPath, portName, operationName)

            functionPrototypeTemplate = this.FunctionPrototypeMap(operationName);
            functionPrototype = sprintf(functionPrototypeTemplate, portName);

            inputArgSpec = this.InputArgSpecMap(operationName);
            outputArgSpec = this.OutputArgSpecMap(operationName);

            set_param(blkPath, 'FunctionPrototype', functionPrototype);
            set_param(blkPath, 'InputArgumentSpecifications', inputArgSpec);
            set_param(blkPath, 'OutputArgumentSpecifications', outputArgSpec);

        end

    end

    methods(Static, Access=public)

        function deepCopyInterface(mdlName, dstInterfacePath) %#ok<INUSD>
            % Copy interface and data types to model

            arxmlPath = fullfile(autosarroot, '+autosar', '+bsw', '+arxml');
            arxmlFiles = {fullfile(arxmlPath, 'AUTOSAR_BaseTypes.arxml')
                fullfile(arxmlPath, 'AUTOSAR_Type.arxml')
                fullfile(arxmlPath, 'AUTOSAR_MOD_PlatformTypes.arxml')
                fullfile(arxmlPath, 'AUTOSAR_FiM.arxml')};

            autosar.bsw.ServiceImplementation.updateReferences(mdlName, arxmlFiles);
        end

        function defaultClientPortName = getDefaultClientPortName()
            defaultClientPortName =  autosar.bsw.FiM_FunctionInhibition.DefaultClientPortName;
        end

        function interfaceName = getInterfaceName()
            interfaceName =  autosar.bsw.FiM_FunctionInhibition.InterfaceName;
        end

        function operations = getOperations()
            operations =  autosar.bsw.FiM_FunctionInhibition.Operations;
        end

        % This block does not expose a Datatype parameter because it does
        % not use any Enum types
        function hastypeParameter = hasDatatypeParameter()
            hastypeParameter = false;
        end

        function dataTypeCallback(~)
        end

        function operationCallback(~)
        end

        function desc = getDescription()
            desc = ['Call an AUTOSAR Function Inhibition Manager (FiM) service function.', ...
                newline, newline, ...
                'Set the Client port name parameter to the port name used by the component for the function call.', ...
                newline, newline, ...
                'Select a FiM operation with the Operation parameter.  After the selection, ', ...
                'the block inputs and outputs correspond to the input and output arguments of the selected operation.'];
        end

        function type = getType()
            type = 'DiagnosticFiM_FunctionInhibitionCaller';
        end

        function help = getHelp()
            help = 'helpview(fullfile(docroot,''ecoder'',''helptargets.map''),''autosar_dem_diagnosticmonitor_block'');';
        end

    end

end

% LocalWords:  Prestore Prestored bsw helptargets FiM diagnosticmonitor dem
