
%   Copyright 2016 The MathWorks, Inc.

classdef NvMService < autosar.bsw.ServiceImplementation
% Class to define NvMServices

    properties(Constant, Access=private)
        DefaultClientPortName = 'NvMService';
        DefaultInterfacePath = '/AUTOSAR/Services/NvM';
        InterfaceName = 'NvMService';
        Operations = {'GetDataIndex'
                      'GetErrorStatus'
                      'EraseNvBlock'
                      'InvalidateNvBlock'
                      'ReadBlock'
                      'RestoreBlockDefaults'
                      'SetDataIndex'
                      'SetRamBlockStatus'
                      'WriteBlock'};

        FunctionPrototypes = {'[DataIndexPtr,ERR] = %s_GetDataIndex()'
            '[RequestResultPtr,ERR] = %s_GetErrorStatus()'
            'ERR = %s_EraseNvBlock()'
            'ERR = %s_InvalidateNvBlock()'
            '[DstPtr,ERR] = %s_ReadBlock()'
            '[DestPtr,ERR] = %s_RestoreBlockDefaults()'
            'ERR = %s_SetDataIndex(DataIndex)'
            'ERR = %s_SetRamBlockStatus(BlockChanged)'
            'ERR = %s_WriteBlock(SrcPtr)'};

        InputArgSpecs = {''
            ''
            ''
            ''
            ''
            ''
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.boolean_spec(true)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'}; % P_NvM_DstPtrType'};

        InputArgSpecTemplate = {''
            ''
            ''
            ''
            ''
            ''
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.boolean_spec(true)'
            '%s'}; % P_NvM_DstPtrType'};

        OutputArgSpecs = {'autosar.bsw.BasicSoftwareCaller.uint8_spec(1), autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1), autosar.bsw.BasicSoftwareCaller.uint8_spec(1)' % P_NvM_RequestResultType, P_Std_ReturnType
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1), autosar.bsw.BasicSoftwareCaller.uint8_spec(1)' % P_NvM_DstPtrType, P_Std_ReturnType
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1), autosar.bsw.BasicSoftwareCaller.uint8_spec(1)' % P_NvM_DstPtrType, P_Std_ReturnType
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'};
        
        FunctionPrototypeMap = containers.Map( autosar.bsw.NvMService.Operations, ...
                                               autosar.bsw.NvMService.FunctionPrototypes );
                                           
        InputArgSpecMap = containers.Map( autosar.bsw.NvMService.Operations, ...
                                          autosar.bsw.NvMService.InputArgSpecs );

        OutputArgSpecMap = containers.Map( autosar.bsw.NvMService.Operations, ...
                                           autosar.bsw.NvMService.OutputArgSpecs );

        OutputArgSpecTemplate = {'autosar.bsw.BasicSoftwareCaller.uint8_spec(1), autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1), autosar.bsw.BasicSoftwareCaller.uint8_spec(1)' % P_NvM_RequestResultType, P_Std_ReturnType
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            '%s, autosar.bsw.BasicSoftwareCaller.uint8_spec(1)' % P_NvM_DstPtrType, P_Std_ReturnType
            '%s, autosar.bsw.BasicSoftwareCaller.uint8_spec(1)' % P_NvM_DstPtrType, P_Std_ReturnType
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'
            'autosar.bsw.BasicSoftwareCaller.uint8_spec(1)'};

        ArgSpecs = {''
                     ''
                     ''
                     ''
                     'uint8(1)'
                     'uint8(1)'
                     ''
                     ''
                     'uint8(1)'};
        
        ArgSpecMaskPromptName = {''
                                  ''
                                  ''
                                  ''
                                  'Argument specification (e.g. uint8(1)):'
                                  'Argument specification (e.g. uint8(1)):'
                                  ''
                                  ''
                                  'Argument specification (e.g. uint8(1)):'};
                  
        ArgSpecMaskPromptNameMap = containers.Map( autosar.bsw.NvMService.Operations, ...
                                                    autosar.bsw.NvMService.ArgSpecMaskPromptName );
                     
        InputArgSpecTemplateMap = containers.Map( autosar.bsw.NvMService.Operations, ...
                                                  autosar.bsw.NvMService.InputArgSpecTemplate );

        OutputArgSpecTemplateMap = containers.Map( autosar.bsw.NvMService.Operations, ...
                                                   autosar.bsw.NvMService.OutputArgSpecTemplate );
        
        ArgSpecMap = containers.Map( autosar.bsw.NvMService.Operations, ...
                                      autosar.bsw.NvMService.ArgSpecs );
    end
    
    methods(Access=public)
        
        function this = NvMService()
        end
    
        function updateFunctionCaller(this, blkPath, portName, operationName)

            functionPrototypeTemplate = this.FunctionPrototypeMap(operationName);
            inputArgSpec = this.InputArgSpecMap(operationName);
            outputArgSpec = this.OutputArgSpecMap(operationName);   
            
            if strcmp(autosar.bsw.NvMService.getArgumentSpecificationVisibility(operationName), 'on')
                argSpec = get_param(blkPath, 'ArgumentSpecification');
                inputArgSpecTemplate = this.InputArgSpecTemplateMap(operationName);
                outputArgSpecTemplate = this.OutputArgSpecTemplateMap(operationName);   

                inputArgSpec = sprintf(inputArgSpecTemplate, argSpec);
                outputArgSpec = sprintf(outputArgSpecTemplate, argSpec);
            end
            
            functionPrototype = sprintf(functionPrototypeTemplate, portName);
            
            set_param(blkPath, 'FunctionPrototype', functionPrototype);
            set_param(blkPath, 'InputArgumentSpecifications', inputArgSpec);
            set_param(blkPath, 'OutputArgumentSpecifications', outputArgSpec);
            
        end
        
    end
    
    methods(Static, Access=public)

        function deepCopyInterface(mdlName, dstInterfacePath) %#ok<INUSD>
        % Copy interface and data types to model
        
            arxmlPath = fullfile(autosarroot, '+autosar', '+bsw', '+arxml');
            arxmlFiles = {fullfile(arxmlPath, 'AUTOSAR_BaseTypes.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_Type.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_MOD_PlatformTypes.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_NvM.arxml')};
            
            autosar.bsw.ServiceImplementation.updateReferences(mdlName, arxmlFiles);
        end
        
        function defaultClientPortName = getDefaultClientPortName()
            defaultClientPortName =  autosar.bsw.NvMService.DefaultClientPortName;
        end
       
        function interfaceName = getInterfaceName()
            interfaceName =  autosar.bsw.NvMService.InterfaceName;
        end
        
        function operations = getOperations()
            operations =  autosar.bsw.NvMService.Operations;
        end

        function dataType = getDefaultArgumentSpecification(~)
            dataType = 'uint8(1)';
        end
        
        % This block does not expose a Datatype parameter because it does
        % not use any Enum types
        function hastypeParameter = hasDatatypeParameter()
            hastypeParameter = false;
        end
        
        function dataTypeCallback(~)
        end
        
        function operationCallback(blkPath)
            operationValue = get_param(blkPath, 'Operation');
            autosar.bsw.ServiceImplementation.argumentSpecificationOperationCallbackImpl(blkPath,...
                 autosar.bsw.NvMService.ArgSpecMap(operationValue),...
                 autosar.bsw.NvMService.ArgSpecMaskPromptNameMap)
        end
        
        % This block exposes a ArgSpec parameter 
        function hasArgSpecParameter = hasArgumentSpecificationParameter()
            hasArgSpecParameter = true;
        end

        % returns true if we should show the ArgSpec parameter for the
        % specified operation
        function visibility = getArgumentSpecificationVisibility(operation)
            datatype = autosar.bsw.NvMService.ArgSpecMap(operation);
            if isempty(datatype)
                visibility = 'off';
            else
                visibility = 'on';
            end
        end
        
        function desc = getDescription()
            desc = autosar.bsw.NvMAdmin.getDescription();
        end
        
        function type = getType()
            type = 'NvMServiceCaller';
        end
        
        function help = getHelp()
            help = 'helpview(fullfile(docroot,''ecoder'',''helptargets.map''),''autosar_nvmservice_block'');';
        end
        
        
    end
    
end

% LocalWords:  Nv MServices Dest bsw MService helptargets nvmservice
