classdef CompiledModelUtils
% utility functions for dealing with a compiled Simulink model
    
%   Copyright 2013-2017 The MathWorks, Inc.

    methods(Static, Access=public)

        function cleanupFuncObj = forceCompiledModel(hModel)
        
            needTerm = false;
            
            fullname = getfullname(hModel);
            if ~autosar.validation.CompiledModelUtils.isCompiled(hModel)
                try
                    feval(fullname, [], [], [], 'compile');
                    needTerm = true;
                catch me
                    if autosar.validation.ExportFcnValidator.isVirtualSubsysExportFcn(hModel)
                        DAStudio.error('RTW:autosar:modelNotCompile', me.message, fullname);
                    else
                        rethrow(me);
                    end
                end
            end
            
            cleanupFuncObj = [];
            if needTerm
                cleanupFuncObj = onCleanup(@()feval(getfullname(hModel), [], [], [], 'term'));
            end
            
            
        end
        
        function cleanupFuncObj = forceCompiledModelForRTW(hModel)
        % Call this function when validating
            needTerm = false;
            
            fullname = getfullname(hModel);
            if ~autosar.validation.CompiledModelUtils.isCompiled(hModel)
                try
                    feval(fullname, [], [], [], 'compileForRTW');
                    needTerm = true;
                catch me
                    if autosar.validation.ExportFcnValidator.isVirtualSubsysExportFcn(hModel)
                        DAStudio.error('RTW:autosar:modelNotCompile', me.message, fullname);
                    else
                        rethrow(me);
                    end
                end
            end
            
            cleanupFuncObj = [];
            if needTerm
                cleanupFuncObj = onCleanup(@()feval(getfullname(hModel), [], [], [], 'term'));
            end
            
            
        end
        
        function isCompiled = isCompiled(hModel)
        
            simStatus = get_param(hModel, 'SimulationStatus');
            isCompiled = strcmpi(simStatus, 'paused') || ...
                strcmpi(simStatus, 'initializing') || ...
                strcmpi(simStatus,'running') || ...
                strcmpi(simStatus,'updating');
        
        end
        
        function wsVar = getReferencedWSVars(hModel, ensureModelCompiled)

            if nargin < 2
                ensureModelCompiled = true;
            end
            
            if ensureModelCompiled
                assert(~strcmp(get_param(hModel,'SimulationStatus'),'stopped'), ...
                    'The model must be compiled');
            end

            mode = 'cached'; % Cached mode is faster but the model must be compiled

            % get list of referenced workspace variables
            modelName = get_param(hModel, 'Name');
            vars = Simulink.findVars(modelName, 'SearchMethod', mode);
            wsVar = [];
            for i=1:length(vars)
                if strcmpi(vars(i).SourceType, 'data dictionary') || ...
                        strcmpi(vars(i).SourceType, 'base workspace')
                    wsVar(end+1).objName = vars(i).Name; %#ok
                    wsVar(end).obj = Simulink.data.internal.getModelGlobalVariable(...
                        modelName, vars(i).Name);
                end
            end
        end 
    end        

    
end

% LocalWords:  Thru
